'''
Contains classes for interactions with teams on NetOrca
'''
from netorca_sdk.base import BaseObj

class TeamsObj(BaseObj):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

